/**
 *
 * 当前应用queen5接受的消息处理。
 *   目前文件依赖 设计为 当前文件 依赖于modules/ediotor/controllers下的文件，这样避免share页面打包
 *   当前模块，导致nats.ws文件被引用，引发 部分手机 share页面打不开的bug
 */

import { queenApi } from "queenjs";

import { AppMsgRecvController } from "@queenjs/assets";
import { nanoid } from "nanoid";
import { useCtx } from "../ctx";
import SaveForm from "./components/SaveForm";
export class AppScreenMsgController extends AppMsgRecvController {
  screenModule: any;
  constructor() {
    super();
  }

  initScreen() {
    this.clearListeners();

    this.addSceneListener("添加场景", async (data) => {
      const asset = data.payload;
      if (data.payload.scenes.length != 1) return false;

      const config = (await queenApi.dialog(
        <SaveForm
          thumbnail={data.payload.scenes[0].thumbnail.url}
          name={data.payload.scenes[0].name}
        />,
        {
          title: "添加场景",
          width: "460px",
          maskClosable: false,
        }
      )) as any;
      console.log("config=>", config);

      const id = nanoid();
      let path = `/assets/${id}`;

      console.log("asset=>", asset);

      const source = data.payload;
      await this.downloadSceneAssetFromHttp(path + "/", source);

      let thumbnail = source.scenes[0].thumbnail;
      if (thumbnail?.url) {
        const thumb = this.loadfile(`/assets/${id}/`, thumbnail.url, `${id}`);
        await thumb?.promise;
        source.scenes[0].thumbnail.url = thumb?.file;
      }

      //拷贝模型到本地磁盘文件
      //压缩文件夹
      const { deviceCtrl } = useCtx();
      const screenDir = `${deviceCtrl.appDataDir}/screen`;

      let isSucc = true;
      // await deviceCtrl.ZipFilesInDir(
      //   screenDir + path,
      //   screenDir + `/assets/${id}.scene`,
      //   (t: string, d: any) => {
      //     console.log("zip dir=====", t, d);
      //     isSucc = t == "succ";
      //   }
      // );

      if (isSucc) {
        const configFile = JSON.stringify(source);
        await deviceCtrl.WriteFileText(
          `${deviceCtrl.appDataDir}/screen/assets/${id}/${id}.json`,
          configFile
        );
        const currentCategory = config.category;
        const assetsJson = await deviceCtrl.ReadFileText(
          `${deviceCtrl.appDataDir}/screen/assets_${currentCategory}.json`
        );
        if (assetsJson.text) {
          const assets = JSON.parse(assetsJson.text);
          const assetItem = {
            _id: id,
            name: config.name,
            thumbnail: thumbnail.url,
            type: "3D",
            source: `/assets/${id}/${id}.json`,
          };
          assets.push(assetItem);
          await deviceCtrl.WriteFileText(
            `${deviceCtrl.appDataDir}/screen/assets_${currentCategory}.json`,
            JSON.stringify(assets)
          );
        }
        this.screenModule.actions.getAssetList();
      }

      // const ok = await deviceCtrl.RemoveDir(screenDir + path);

      return true;
    });

    this.emitChange();
  }

  loadfile(dir: string, url: string, cusName = "") {
    if (!url) return;
    const { deviceCtrl } = useCtx();
    let names = url.split("/");
    let name = names[names.length - 1];
    if (cusName) {
      const index = name.lastIndexOf(".");
      name = cusName + name.substring(index);
    }
    const screenDir = deviceCtrl.appDataDir + "/screen";
    const filepath = screenDir + dir + name;
    return {
      file: dir + name,
      promise: new Promise((resolve) => {
        deviceCtrl.DownloadFile(url, filepath).then((ok) => {
          setTimeout(() => {
            resolve({ name: name, ok: ok });
          }, 0);
        });
      }),
    };
  }

  async downloadSceneAssetFromHttp(
    path: string,
    source: {
      mats: any[];
      geoms: any[];
      products: any[];
      scenes: any[];
      env3ds: any[];
    }
  ) {
    queenApi.showLoading("资源下载中");

    let files: any = [];

    const downHttp = (wkPath: string, url: string, name = "") => {
      return this.loadfile(wkPath, url, name);
    };

    this.downGeoms(path, source.geoms, files, downHttp);

    this.downMats(path, source.mats, files, downHttp);

    this.downEnv3ds(path, source.env3ds, files, downHttp);

    //场景背景
    source.scenes.forEach((s) => {
      const bgUrl = s.background?.image?.url;
      if (bgUrl) {
        const f = downHttp(path, bgUrl, "bg");
        if (f) {
          s.background.image.url = f.file;
          files.push(f.promise);
        }
      }
    });
    const result = await Promise.all(files);
    queenApi.hideLoading();
    const failRes = result.filter((e) => {
      return e.ok == false;
    });
    console.log(result, failRes);
    return source;
  }
  downMats(
    path: string,
    mats: any[],
    files: any[],
    handleFile: (wkpath: string, url: string, name?: string) => any
  ) {
    //材质处理
    mats.forEach((mItem) => {
      const abf = handleFile(path, mItem.albedo.texture?.url);
      //base
      if (abf) {
        files.push(abf?.promise);
        mItem.albedo.texture.url = abf.file;
      }
      //法线
      if (mItem?.normal?.texture) {
        const norf = handleFile(path, mItem.normal.texture?.url);
        if (norf) {
          files.push(norf?.promise);
          mItem.normal.texture.url = norf.file;
        }
      }
      //粗糙度
      if (mItem?.roughness?.texture) {
        const rouf = handleFile(path, mItem.roughness.texture?.url);
        if (rouf) {
          files.push(rouf?.promise);
          mItem.roughness.texture.url = rouf.file;
        }
      }
      //金属度
      if (mItem?.metalness?.texture) {
        const metaf = handleFile(path, mItem.metalness.texture?.url);
        if (metaf) {
          files.push(metaf?.promise);
          mItem.metalness.texture.url = metaf.file;
        }
      }
      if (mItem?.opacity?.texture) {
        const opaf = handleFile(path, mItem.opacity.texture?.url);
        if (opaf) {
          files.push(opaf?.promise);
          mItem.opacity.texture.url = opaf.file;
        }
      }
    });
  }

  //模型处理
  downGeoms(
    path: string,
    geoms: any[],
    files: any[],
    handleFile: (wkpath: string, url: string, name?: string) => any
  ) {
    geoms.map((geom) => {
      const osgjsUrl = geom.osgjs.url;
      let osjsf = handleFile(path, osgjsUrl);
      if (osjsf) {
        files.push(osjsf?.promise);
        geom.osgjs.url = osjsf?.file;
      }

      const osgbinUrl = osgjsUrl.substring(0, osgjsUrl.length - 5) + "bin";
      let binf = handleFile(path, osgbinUrl);
      if (binf) {
        files.push(binf?.promise);
      }
      if (geom?.shadow?.url) {
        let shadowf = handleFile(path, geom.shadow.url);
        if (shadowf) {
          files.push(shadowf.promise);
          geom.shadow.url = shadowf.file;
        }
      }
    });
  }

  downEnv3ds(
    path: string,
    env3ds: any[],
    files: any[],
    handleFile: (wkpath: string, url: string, name?: string) => any
  ) {
    //环境球
    const wkenv3d = path + "env3d/";

    env3ds.forEach((defultEnv3d: any) => {
      const envPath = `${wkenv3d}${defultEnv3d.id}/`;
      if (defultEnv3d.background?.image?.url) {
        let f = handleFile(
          envPath,
          defultEnv3d.background?.image?.url,
          "background"
        );
        if (f) {
          defultEnv3d.background.image.url = f.file;
          files.push(f.promise);
        }
      }

      defultEnv3d.config.textures.forEach((t: any) => {
        t.images.forEach((im: any) => {
          const f = handleFile(envPath, im.file);
          if (f) {
            im.file = f.file;
            files.push(f.promise);
          }
        });
      });
    });
  }
}
