import { css } from "@linaria/core";

import { defineComponent, onMounted, reactive, ref, watch } from "vue";

import { object } from "vue-types";

import { LoadingOutlined } from "@ant-design/icons-vue";
import { Spin } from "ant-design-vue";

import { useQueditor } from "@queenjs-modules/queditor";

import { useList } from "@/modules/list";
import { cloneDeep } from "lodash";
import { useCtx } from "@/comm/ctx";

function getQuery() {
  const search = location.href.split("?")[1];
  const ret: { [name: string]: string } = {};
  if (search) {
    const aParams = search.split("&");
    const len = aParams.length;
    for (let i = 0; i < len; i++) {
      const aParam = aParams[i].split("=");
      const sParamName = decodeURIComponent(aParam[0]);
      const sParamValue = decodeURIComponent(aParam[1]);
      ret[sParamName] = sParamValue;
    }
  }
  return ret;
}
const Canvas3d = defineComponent({
  props: {
    pack: object<any>(),
  },
  emits: ["init"],
  setup(props, { emit }) {
    const state = reactive({ loaded: false });
    const { actions, components, controls, store } = useQueditor();
    store.showLoading = false;
    const getLocalSource = (asset: any) => {
      let source = cloneDeep(asset);

      const { deviceCtrl } = useCtx();
      const httpServer = deviceCtrl.httpServer;

      //模型处理
      let geom = source.geoms[0];

      geom.osgjs.url = httpServer + geom.osgjs.url;
      source.geoms = [geom];
      if (geom.shadow.url) {
        geom.shadow.url = httpServer + geom.shadow.url;
        // files.push(f.promise);
      }

      //材质处理
      source.mats.forEach((m: any) => {
        //base map

        m.albedo.texture.url = httpServer + m.albedo.texture.url;

        //法线
        if (m.normal?.texture) {
          m.normal.texture.url = httpServer + m.normal.texture.url;
        }
        //粗糙度
        if (m.roughness.texture) {
          m.roughness.texture.url = httpServer + m.roughness.texture.url;
        }

        //金属度
        if (m.metalness.texture) {
          m.metalness.texture.url = httpServer + m.metalness.texture.url;
        }

        //透明度
        if (m.opacity.texture) {
          m.opacity.texture.url = httpServer + m.opacity.texture.url;
        }
      });

      //环境球
      source.env3ds.forEach((item: any) => {
        if (item.background?.image?.url) {
          item.background.image.url = httpServer + item.background.image.url;
        }
        item.config.textures.forEach((t: any) => {
          t.images.forEach((im: any) => {
            im.file = httpServer + im.file;
          });
        });
      });

      source.scenes.forEach((c: any) => {
        //@ts-ignore
        const url = c.background?.image?.url;
        if (url) {
          //@ts-ignore
          c.background.image.url = httpServer + c.background.image.url;
        }
      });

      return source;
    };
    const initQueen = () => {
      const source = getLocalSource(props.pack);
      actions.initPack(source);
      actions.on("initQueen3dScene:success", () => {
        const queen3d = controls.queen3dCtrl.queen3d;
        const app = controls.queen3dCtrl.queen3d.getAppInstance();

        app.setNoPan(true);

        queen3d.setTransparntBg();

        queen3d.setControllerZoom(0.5, 6);

        if (queen3d.picker) {
          queen3d.picker.enable = false;
        }

        queen3d.redraw();
        state.loaded = true;
        emit("init", true);
      });
    };
    initQueen();

    return () => (
      <components.Viewport.Canvas
        class={`${CanvasStyle} ${state.loaded ? "loaded" : null}`}
      />
    );
  },
});

export default defineComponent({
  setup(props) {
    const query = getQuery();
    const dialogId = query.dialogid;
    const source = query.source;
    const packRef = ref();
    const state = reactive({ loaded: false, loading: true });
    // window.addEventListener("message", (e: any) => {
    //   try {
    //     const msgData = JSON.parse(e.data);
    //     if (msgData.type == "data") {
    //       packRef.value = msgData.data;
    //       state.loaded = true;
    //     }
    //   } catch (e) {
    //     console.log(e);
    //   }
    // });
    const { actions } = useList();
    onMounted(async () => {
      const res = await actions.loadPack3d({ source: source } as any);
      packRef.value = res?.data;
      state.loaded = true;
      document.addEventListener("mousedown", () => {
        window.parent.postMessage(JSON.stringify({ type: "start", dialogId }));
      });
      document.addEventListener("touchstart", () => {
        window.parent.postMessage(JSON.stringify({ type: "start", dialogId }));
      });
      document.addEventListener("mouseup", () => {
        window.parent.postMessage(JSON.stringify({ type: "end", dialogId }));
      });
      document.addEventListener("touchend", () => {
        window.parent.postMessage(JSON.stringify({ type: "end", dialogId }));
      });
      window.parent.postMessage(JSON.stringify({ type: "ready" }));
    });

    return () => {
      return (
        <>
          {state.loaded && (
            <Canvas3d
              pack={packRef.value}
              onInit={(v) => {
                state.loading = false;
              }}
            />
          )}

          {state.loading && (
            <Spin
              class={loadingStyle}
              indicator={
                <LoadingOutlined style={{ fontSize: "40px", color: "#fff" }} />
              }
            />
          )}
        </>
      );
    };
  },
});

// const rootStyle = css`
//   padding: 30px;
//   background-color: red;
// `;

const loadingStyle = css`
  position: absolute;
  top: 50%;
  left: 50%;
  transform: translate(-50%, -50%);
`;
const CanvasStyle = css`
  width: 100%;
  height: 100%;
  visibility: hidden;
  &.loaded {
    visibility: inherit;
  }
`;
