import { useCtx } from "@/comm/ctx";
import { useList } from "@/modules/list";
import { css } from "@linaria/core";
import { Image } from "@queenjs/ui";
import { Button, Select, Space, Table } from "ant-design-vue";

import { queenApi } from "queenjs";
import { defineComponent, reactive } from "vue";
import CategoryModal from "./categoryModal";

export default defineComponent({
  setup() {
    const ctx = useList();
    const { store, showModal, actions } = ctx;
    const { deviceCtrl, storeCtrl } = useCtx();
    const state = reactive({
      select: "default",
    });
    const del = async (item: any) => {
      await queenApi.showConfirm({
        type: "danger",
        content: `确认删除${item?.name || ""}？`,
      });
      actions.delAsset(item);
    };

    const columns = [
      {
        title: "封面图",
        dataIndex: "thumbnail",
        key: "thumbnail",
        customRender: ({ text }: any) => {
          return (
            <Image
              class="thumbnail"
              src={deviceCtrl.httpServer + text}
              size={0}
            />
          );
        },
      },
      {
        title: "名称",
        dataIndex: "name",
        key: "name",
      },
      {
        title: "操作",
        key: "action",
        dataIndex: "action",
        customRender: ({ record }: any) => {
          return (
            <Space>
              <Button
                type="link"
                danger
                onClick={() => {
                  del(record);
                }}
              >
                删除
              </Button>
            </Space>
          );
        },
      },
    ];
    actions.getAssetList();
    const initShowCategory = async () => {
      const currentCategory = localStorage.getItem("category") || "default";
      state.select = currentCategory;
    };
    const { msgCtrl } = useCtx();
    msgCtrl.screenModule = ctx;
    initShowCategory();
    msgCtrl.initScreen();
    const setShowCategory = (value: any) => {
      localStorage.setItem("category", value || "default");
      state.select = value;
      actions.getAssetList();
    };
    const createShowCategory = async () => {
      const { deviceCtrl, storeCtrl } = useCtx();
      const res: any = await showModal(<CategoryModal />, {
        title: "新建分类",
      });
      const options = [...storeCtrl.state.category];
      const optionItem = options.find((e) => {
        return e.value == res.value;
      });
      if (optionItem) {
        return;
      }
      options.push(res);
      storeCtrl.state.category = options;
      const appDataDir = deviceCtrl.appDataDir;
      await deviceCtrl.WriteFileText(
        `${appDataDir}/screen/category.json`,
        JSON.stringify(options)
      );
    };
    return () => (
      <div class={ViewStyle}>
        <div class="table_header">
          <div class="title">展示列表</div>
          <div>
            <Space>
              <Select
                class={"select"}
                value={state.select}
                options={storeCtrl.state.category}
                onChange={setShowCategory}
              ></Select>
              <Button
                type="primary"
                ghost
                onClick={() => {
                  createShowCategory();
                }}
              >
                新建展示分类
              </Button>
            </Space>
          </div>
        </div>
        <Table
          size="small"
          columns={columns}
          dataSource={store.list}
          rowKey={(record) => record?._id}
        ></Table>
      </div>
    );
  },
});

const ViewStyle = css`
  padding: 20px;
  .table_header {
    display: flex;
    align-items: center;
    justify-content: space-between;
    margin-bottom: 20px;
    .title {
      font-size: 18px;
      font-weight: bold;
    }
    .select {
      width: 150px;
    }
  }
  .ant-table-thead > tr > th,
  .ant-table-tbody > tr > td {
    text-align: center;
  }
  .thumbnail {
    width: 120px;
    object-fit: contain;
  }
`;
